# this is not a function file
# prevents

function file_header = loadJazzFile( file_name , load_signals_data = 0 , signals_to_load = int32([]) , calibrate_signals_data = 0 )
  f = fopen( file_name , 'rb' , 'ieee-be' );
  
  magic_num_1 = fread( f , 1 , '*uint8' , 'ieee-be' );
  magic_num_2 = fread( f , 1 , '*uint8' , 'ieee-be' );
  magic_num_3 = fread( f , 1 , '*uint8' , 'ieee-be' );
  magic_num_4 = fread( f , 1 , '*uint8' , 'ieee-be' );
  
  if ( magic_num_1 != 0xF9 || magic_num_2 != 0xD7 || magic_num_3 != 0x2C || magic_num_4 != 0x0A )
    error ( 'This is not a valid jazz data file (incorrect magic number)!' );
    return;
  endif
  
  header_version = fread( f , 1 , '*uint32' , 'ieee-be' );
  
  res = header_version;
  
  if ( header_version < 6 || header_version > 12 )
    error( "Header version %d not supported!" , header_version );
    return;
  endif
	
	printf( "Loading file header...\n" )
  
  file_header.header_version = header_version;
  
  file_header.header_size = fread( f , 1 , '*uint32' , 'ieee-be' );

  file_header.app_name = transpose( fread( f , 128 , '*char*1' ) );
  file_header.app_name = file_header.app_name(1:(find(file_header.app_name==0,1)-1));

  file_header.user_info = transpose( fread( f , 1024 , '*char*1' ) );
  file_header.user_info = file_header.user_info(1:(find(file_header.user_info==0,1)-1));

  if ( header_version >= 12 )
    file_header.tech_info = transpose( fread( f , 1024 , '*char*1' ) );
    file_header.tech_info = file_header.tech_info(1:(find(file_header.tech_info==0,1)-1));
  endif

  file_header.day = fread( f , 1 , '*uint8' , 'ieee-be' );
  file_header.month = fread( f , 1 , '*uint8' , 'ieee-be' );
  file_header.year = fread( f , 1 , '*uint16' , 'ieee-be' );

  file_header.hour = fread( f , 1 , '*uint8' , 'ieee-be' );
  file_header.minute = fread( f , 1 , '*uint8' , 'ieee-be' );
  file_header.second = fread( f , 1 , '*uint8' , 'ieee-be' );
  
  if ( header_version >= 11 )
    file_header.millisecond = fread( f , 1 , '*uint16' , 'ieee-be' );
  endif

  file_header.signal_count = fread( f , 1 , '*uint8' , 'ieee-be' );
  
  for i = 1:file_header.signal_count
    file_header.signals(i) = loadSignalHeader( f , header_version );
  end
  
  file_header.packet_count = fread( f , 1 , '*uint32' , 'ieee-be' );
  
  printf( 'File header loaded sucessfully. Found %d signals.\n' , file_header.signal_count );
  
  if ( load_signals_data )
    current_sample = ones( 1 , file_header.signal_count , 'int32' );
    
    if ( length(signals_to_load) == 0 )
      signals_to_load = 1:file_header.signal_count;
    endif

    load_signal = zeros( 1 , file_header.signal_count , 'int8' );
    
    printf( "Signals selected to load data:\n" );
    
    data_size = 0;
    
    sig_read_samples = zeros( 1 , file_header.signal_count , 'int32' );
    sig_read_bytes = zeros( 1 , file_header.signal_count , 'int32' );
    sig_read_precision = cell( file_header.signal_count , 1 );
    
    for sig = 1:file_header.signal_count
      if length( find( signals_to_load == sig , 1 ) ) == 1
        printf( " -> %s\n" , file_header.signals(sig).full_name );
        file_header.signals(sig).data = zeros( 1 , file_header.packet_count * file_header.signals(sig).sample_count );
        load_signal(sig) = 1;
      endif
      
      data_size = data_size + ( file_header.signals(sig).sample_count * file_header.signals(sig).sample_size );
      
      sig_read_samples(sig) = file_header.signals(sig).sample_count;
      
      sig_read_bytes(sig) = file_header.signals(sig).sample_count * file_header.signals(sig).sample_size;
      
      if file_header.signals(sig).value_type == 1 # signed integer
        if file_header.signals(sig).sample_size == 1
            sig_read_precision{sig} = 'int8';
        elseif file_header.signals(sig).sample_size == 2
            sig_read_precision{sig} = 'int16';
         elseif file_header.signals(sig).sample_size == 4
            sig_read_precision{sig} = 'int32';
         elseif file_header.signals(sig).sample_size == 8
            sig_read_precision{sig} = 'int64';
         else
            printf( "Unsupported sample size %d for signal %d\n" , file_header.signals(sig).sample_size , file_header.signals(sig).full_name );
            return;
         endif
      elseif file_header.signals(sig).value_type == 2 # unsigned integer
        if file_header.signals(sig).sample_size == 1
            sig_read_precision{sig} = 'uint8';
        elseif file_header.signals(sig).sample_size == 2
            sig_read_precision{sig} = 'uint16';
         elseif file_header.signals(sig).sample_size == 4
            sig_read_precision{sig} = 'uint32';
         elseif file_header.signals(sig).sample_size == 8
            sig_read_precision{sig} = 'uint64';
         else
            printf( "Unsupported sample size %d for signal %d\n" , file_header.signals(sig).sample_size , file_header.signals(sig).full_name );
            return;
         endif
      elseif file_header.signals(sig).value_type == 3 # float64
         if file_header.signals(sig).sample_size == 4
            sig_read_precision{sig} = 'float32';
         elseif file_header.signals(sig).sample_size == 8
            sig_read_precision{sig} = 'float64';
         else
            printf( "Unsupported sample size %d for signal %d\n" , file_header.signals(sig).sample_size , file_header.signals(sig).full_name );
            return;
         endif
      else
        printf( "Unsupported value type %d for signal %d\n" , file_header.signals(sig).value_type , file_header.signals(sig).full_name );
        return;
      endif
      
    endfor
    
    #printf( "Packet size %d bytes\n" , data_size );
    
    data_size = data_size * file_header.packet_count;
    
    #printf( "Data size %d bytes\n" , data_size );
    
    one_tenth = int32( file_header.packet_count / 10 );
    
    printf( "Loading signals data... ( 0%%)\n" );
    
    if exist( 'OCTAVE_VERSION' , 'builtin' )
      fflush(stdout);
    endif
    
    #raw_data = fread( f , data_size , '*uint8' , 0 , 'ieee-be' );
    #raw_data_ptr = 1;
    
    for pkt = 1:file_header.packet_count
      
      for sig = 1:file_header.signal_count
      
        if load_signal(sig)
        
          file_header.signals(sig).data(current_sample(sig):(current_sample(sig)+sig_read_samples(sig)-1)) = fread( f , sig_read_samples(sig) , sig_read_precision{sig} , 0 , 'ieee-be' );
          #file_header.signals(sig).data(current_sample(sig):(current_sample(sig)+sig_read_samples(sig)-1)) = swapbytes( typecast( raw_data(raw_data_ptr:(raw_data_ptr+sig_read_bytes(sig)-1)) , sig_read_precision{sig} ) );
          current_sample(sig) += sig_read_samples(sig);
                    
        else
        
          fseek( f , sig_read_bytes(sig) , 0 ); 
                    
        endif
        
        #raw_data_ptr += sig_read_bytes(sig);
        
      endfor
      
      if mod(pkt,one_tenth) == 0
        printf( "Loading signals data... (%d%%)\n" , round( 100 * pkt / file_header.packet_count ) );
        if exist( 'OCTAVE_VERSION' , 'builtin' )
          fflush(stdout);
        endif
      endif
      
    endfor
    
    printf( "Sucessfully loaded %d signals!\n" , length(signals_to_load) );
    
    if ( calibrate_signals_data )
      printf( "Calibrating signals data...\n" );
      
      for sig = 1:file_header.signal_count
        if ( load_signal(sig) )
          if ( file_header.signals(sig).is_calib )
            file_header.signals(sig).data -= file_header.signals(sig).calib_offset;
            file_header.signals(sig).data /= file_header.signals(sig).calib_factor;
          endif
        endif
      endfor
      
      printf( "Signals data calibrated.\n" );
    endif
    
  endif

  printf( "Done!\n" );
  
  fclose( f );
  
endfunction



function signal_header = loadSignalHeader( f , header_version )

  signal_header.full_name = transpose( fread( f , 128 , '*char*1' ) );
  signal_header.full_name = signal_header.full_name(1:(find(signal_header.full_name==0,1)-1));
  
  signal_header.short_name = transpose( fread( f , 32 , '*char*1' ) );
  signal_header.short_name = signal_header.short_name(1:(find(signal_header.short_name==0,1)-1));

 	signal_header.freq_period = fread( f , 1 , '*uint32' , 'ieee-be' );
  signal_header.freq_samples = fread( f , 1 , '*uint32' , 'ieee-be' );

  if ( header_version >= 9 )
    signal_header.value_type = fread( f , 1 , '*uint32' , 'ieee-be' );
    signal_header.sample_count = fread( f , 1 , '*uint32' , 'ieee-be' );
    signal_header.sample_size = fread( f , 1 , '*uint32' , 'ieee-be' );
	else
    signal_header.value_type = fread( f , 1 , '*uint8' , 'ieee-be' );
    signal_header.sample_count = fread( f , 1 , '*uint8' , 'ieee-be' );
    signal_header.sample_size = fread( f , 1 , '*uint8' , 'ieee-be' );
  end

  signal_header.calib_factor = fread( f , 1 , '*float64' , 'ieee-be' );

	if ( header_version <= 7 )
		if ( signal_header.calib_factor < 1.0 )
			if ( signal_header.calib_factor > 0.0000001 )
				signal_header.calib_factor = 1.0 / signal_header.calib_factor;
       end
		end
	end
  
  signal_header.calib_unit = transpose( fread( f , 64 , '*char*1' ) );
  signal_header.calib_unit = signal_header.calib_unit(1:(find(signal_header.calib_unit==0,1)-1));
  
  if ( header_version >= 7 )
    signal_header.is_calib = fread( f , 1 , '*uint8' , 'ieee-be' );
	else
	  signal_header.is_calib = length(signal_header.calib_unit) > 0;
	end
  
	if ( header_version >= 10 )
    signal_header.calib_offset = fread( f , 1 , '*float64' , 'ieee-be' );
	else
    signal_header.calib_offset = 0.0;
  end

endfunction;
