loadSignalHeader <- function( f , header_version )
{
  if ( header_version >= 6 )
  {
    sig_full_name = readChar( f , 128 )
    sig_short_name = readChar( f , 32 )
  }
  else
  {
    sig_full_name_len = readBin( f , integer() , size = 4 , endian = "big" )
    sig_full_name = readChar( f , s?readfig_full_name_len )

    sig_short_name_len = readBin( f , integer() , size = 4 , endian = "big" )
    sig_short_name = readChar( f , sig_short_name_len )
  }

  if ( header_version >= 5 )
  {
  	freq_period = readBin( f , integer() , size = 4 , endian = "big" )
  }
  else
  {
    freq_period = 1000;
  }

  freq_samples = readBin( f , integer() , size = 4 , endian = "big" )

  if ( header_version >= 9 )
	{
    value_type = readBin( f , integer() , size = 4 , endian = "big" )
    sample_count = readBin( f , integer() , size = 4 , endian = "big" )
    sample_size = readBin( f , integer() , size = 4 , endian = "big" )
	}
	else
	{
    value_type = readBin( f , integer() , size = 1 , signed = FALSE , endian = "big" )
    sample_count = readBin( f , integer() , size = 1 , signed = FALSE , endian = "big" )
    sample_size = readBin( f , integer() , size = 1 , signed = FALSE , endian = "big" )
	}

  if ( header_version >= 4 )
	{
    calib_factor = readBin( f , numeric() , size = 8 , endian = "big" )

		if ( header_version <= 7 )
		{
			if ( calib_factor < 1.0 )
			{
				if ( calib_factor > 0.0000001 )
				{
					calib_factor = 1.0 / calib_factor;
				}
			}
		}

		if ( header_version >= 6 )
		{
      calib_unit = readChar( f , 64 )
		}
		else
		{
      calib_unit_len = readBin( f , integer() , size = 4 , endian = "big" )
      calib_unit = readChar( f , sig_calib_unit_len )
		}
	}
	else
	{
		calib_factor = 1.0;
		calib_unit = "";
	}

	if ( header_version >= 7 )
	{
    is_calib = readBin( f , integer() , size = 1 , signed = FALSE , endian = "big" )
	}
	else
	{
    is_calib = nchar(sig_calib_unit) > 0
	}

	if ( header_version >= 10 )
	{
    calib_offset = readBin( f , numeric() , size = 8 , endian = "big" )
	}
	else
	{
    calib_offset = 0.0
	}

  res = list( full_name=sig_full_name , short_name=sig_short_name , freq_period=freq_period , freq_samples=freq_samples , value_type=value_type , sample_count=sample_count , sample_size=sample_size , is_calib=is_calib , calib_factor=calib_factor , calib_offset=calib_offset , calib_unit=calib_unit )
  
  return(res)
}

loadJazzFile <- function( file_name , load_signals_data = 0 , signals_to_load = vector() , calibrate_signals_data = 0 )
{
  f = file( file_name , "rb" )
  
	magic_num_1 <- readBin( f , integer() , size = 2 , signed = FALSE , endian = "big" )
	magic_num_2 <- readBin( f , integer() , size = 2 , signed = FALSE , endian = "big" )
	#print(as.hexmode(magic_num_1))
	#print(as.hexmode(magic_num_2))
	if ( magic_num_1 != as.integer(0xF9D7) || magic_num_2 != as.integer(0x2C0A) )
  {
    stop( "This is not a valid jazz data file (magic number is incorrect)!" )
	}
	
	header_version <- readBin( f , integer() , size = 4 , endian = "big" )
	
	if ( header_version < 6 || header_version > 12 )
	{
    stop( paste( "Header version " , header_version , " not supported" , "!\n" , sep = "" , collapse="" ) )
	}
	
	cat( "Loading file header...\n" )
	
	file_header <- list()
	
	file_header$header_version <- header_version

	file_header$header_size <- readBin( f , integer() , size = 4 , endian = "big" )

  file_header$app_name <- readChar( f , 128 )

  file_header$user_info <- readChar( f , 1024 )

  if ( header_version >= 12 )
  {
    file_header$tech_info <- readChar( f , 1024 )
  }

  file_header$day <- readBin( f , integer() , size = 1 , signed = FALSE , endian = "big" )
  file_header$month <- readBin( f , integer() , size = 1 , signed = FALSE , endian = "big" )
  file_header$year <- readBin( f , integer() , size = 2 , signed = FALSE , endian = "big" )

  file_header$hour <-readBin( f , integer() , size = 1 , signed = FALSE , endian = "big" )
  file_header$minute <- readBin( f , integer() , size = 1 , signed = FALSE , endian = "big" )
  file_header$second <- readBin( f , integer() , size = 1 , signed = FALSE , endian = "big" )
  
  if ( header_version >= 11 )
  {
    file_header$millisecond <- readBin( f , integer() , size = 2 , signed = FALSE , endian = "big" )
  }

  file_header$signal_count <- readBin( f , integer() , size = 1 , signed = FALSE , endian = "big" )

  signals = list()

  for ( i in 1:file_header$signal_count )
  {
    s <- loadSignalHeader( f , header_version )
    signals[[length(signals)+1]] <- s
    names(signals)[length(signals)] <- s$short_name
  }
  
  file_header$packet_count <- readBin( f , integer() , size = 4 , endian = "big" )

  file_header$signals <- signals
  
  cat( "File header loaded successfully. Found" , file_header$signal_count , "signals.\n" )
  
  if ( load_signals_data )
  {
    current_sample <- rep( 1 , file_header$signal_count )
    
    if ( length(signals_to_load) == 0 )
    {
      signals_to_load <- seq(1,file_header$signal_count)
    }
    
    load_signal <- rep( 0 , file_header$signal_count )
    
    cat("Signals selected to load data:\n")
    
    data_size = 0
    
    for ( sig in 1:file_header$signal_count )
    {
      if ( sig %in% signals_to_load )
      {
        cat( " -> " , file_header$signals[[sig]]$full_name , "\n" )
        file_header$signals[[sig]]$data <- rep( NA_integer_ , file_header$packet_count * file_header$signals[[sig]]$sample_count )
        load_signal[sig] = 1
      }
      
      data_size <- data_size + ( file_header$signals[[sig]]$sample_count * file_header$signals[[sig]]$sample_size )
    }
    
    #cat( "Packet size" , data_size , "bytes\n" )
    
    data_size <- data_size * file_header$packet_count
    
    #cat( "Data size" , data_size , "bytes\n" )
    
    raw_data = readBin( f , raw() , n = data_size , size = 1 , endian = "big" )
    
    close(f)
    
    raw_data_ptr = 0

    one_tenth = as.integer( file_header$packet_count / 10 );
    
    cat( "Loading signals data... (" , 0 , "%)" , "\n" , sep = "" )
    
    for ( pkt in 1:file_header$packet_count )
    {
      for ( sig in 1:file_header$signal_count )
      {
        size <- file_header$signals[[sig]]$sample_count * file_header$signals[[sig]]$sample_size
        
        if ( load_signal[sig] == 1 )
        {
          if ( file_header$signals[[sig]]$value_type == 1 ) # signed integer
          {
            if ( file_header$signals[[sig]]$sample_size < 4 )
            {
              file_header$signals[[sig]]$data[current_sample[sig]:(current_sample[sig]+file_header$signals[[sig]]$sample_count-1)] <- readBin( raw_data[(1:size)+raw_data_ptr] , integer() , n = file_header$signals[[sig]]$sample_count , size = file_header$signals[[sig]]$sample_size , signed = TRUE , endian = "big" )
              current_sample[sig] <- current_sample[sig] + file_header$signals[[sig]]$sample_count
            }
            else
            {
              file_header$signals[[sig]]$data[current_sample[sig]:(current_sample[sig]+file_header$signals[[sig]]$sample_count-1)] <- readBin( raw_data[(1:size)+raw_data_ptr] , integer() , n = file_header$signals[[sig]]$sample_count , size = file_header$signals[[sig]]$sample_size , endian = "big" )
              current_sample[sig] <- current_sample[sig] + file_header$signals[[sig]]$sample_count
            }
          }
          else if ( file_header$signals[[sig]]$value_type == 2 ) # unsigned integer
          {
            if ( file_header$signals[[sig]]$sample_size < 4 )
            {
              file_header$signals[[sig]]$data[current_sample[sig]:(current_sample[sig]+file_header$signals[[sig]]$sample_count-1)] <- readBin( raw_data[(1:size)+raw_data_ptr] , integer() , n = file_header$signals[[sig]]$sample_count , size = file_header$signals[[sig]]$sample_size , signed = FALSE , endian = "big" )
              current_sample[sig] <- current_sample[sig] + file_header$signals[[sig]]$sample_count
            }
            else
            {
              file_header$signals[[sig]]$data[current_sample[sig]:(current_sample[sig]+file_header$signals[[sig]]$sample_count-1)] <- readBin( raw_data[(1:size)+raw_data_ptr] , integer() , n = file_header$signals[[sig]]$sample_count , size = file_header$signals[[sig]]$sample_size , endian = "big" )
              current_sample[sig] <- current_sample[sig] + file_header$signals[[sig]]$sample_count
            }
          }
          else if ( file_header$signals[[sig]]$value_type == 3 ) # numerical
          {
            file_header$signals[[sig]]$data[current_sample[sig]:(current_sample[sig]+file_header$signals[[sig]]$sample_count-1)] <- readBin( raw_data[(1:size)+raw_data_ptr] , double() , n = file_header$signals[[sig]]$sample_count , size = file_header$signals[[sig]]$sample_size , endian = "big" )
            current_sample[sig] <- current_sample[sig] + file_header$signals[[sig]]$sample_count
          }
          else
          {
            stop( cat( "Unsupported value type" , file_header$signals[[sig]]$value_type , "for signal" , file_header$signals[[sig]]$full_name , "\n" ) )
          }
        }

        raw_data_ptr <- raw_data_ptr + size
      }
      
      if ( pkt %% one_tenth == 0 )
      {
        cat( "Loading signals data... (" , round( 100 * pkt / file_header$packet_count ) , "%)" , "\n" , sep = "" )
      }
    }
    
    cat("Sucessfully loaded",length(signals_to_load),"signals!\n")
    
    if ( calibrate_signals_data )
    {
      cat( "Calibrating signals data...\n" )
      
      for ( sig in 1:file_header$signal_count )
      {
        if ( load_signal[sig] == 1 )
        {
          if ( file_header$signals[[sig]]$is_calib )
          {
            file_header$signals[[sig]]$data <- file_header$signals[[sig]]$data - file_header$signals[[sig]]$calib_offset
            file_header$signals[[sig]]$data <- file_header$signals[[sig]]$data / file_header$signals[[sig]]$calib_factor
          }
        }
      }
      
      cat( "Signals data calibrated.\n" )
    }
  }

  cat("Done!")
  
  return(file_header)
}

loadJazzSignal <- function( file_name , signal_index , calibrate_signal_data = 0 )
{
  if ( mode(signal_index) != "numeric" )
  {
    stop( "Parameter \"signal_index\" must be a numeric type!" )
  }
  
  if ( length(signal_index) != 1 )
  {
    stop( "Parameter \"signal_index\" must contains only one value!" )
  }
  
  return ( loadJazzFile( file_name , 1 , round(signal_index) , calibrate_signal_data )$signals[[signal_index]]$data )
}
